# Summary

[Introduction](./introduction.md)
[Quick Start](./user/quick-start.md)
[Quick Start Operator](./user/quick-start-operator.md)
[Concepts](./user/concepts.md)
[Manifesto](./user/manifesto.md)
[Personas](./user/personas.md)
- [Tasks](./tasks/index.md)
    - [Certificate Management](./tasks/certs/index.md)
        - [Using Custom Certificates](./tasks/certs/using-custom-certificates.md)
        - [Generating a Kubeconfig](./tasks/certs/generate-kubeconfig.md)
        - [Auto Rotate Certificates in KCP](./tasks/certs/auto-rotate-certificates-in-kcp.md)
    - [Bootstrap](./tasks/bootstrap/index.md)
        - [Kubeadm based bootstrap](./tasks/bootstrap/kubeadm-bootstrap/index.md)
            - [Kubelet configuration](./tasks/bootstrap/kubeadm-bootstrap/kubelet-config.md)
        - [MicroK8s based bootstrap](./tasks/bootstrap/microk8s-bootstrap.md)
    - [Upgrading management and workload clusters](./tasks/upgrading-clusters.md)
    - [External etcd](./tasks/external-etcd.md)
    - [Using kustomize](./tasks/using-kustomize.md)
    - [Upgrading Cluster API components](./tasks/upgrading-cluster-api-versions.md)
    - [Control plane management](./tasks/control-plane/index.md)
        - [Kubeadm based control plane management](./tasks/control-plane/kubeadm-control-plane.md)
        - [MicroK8s based control plane management](./tasks/control-plane/microk8s-control-plane.md)
    - [Updating Machine Infrastructure and Bootstrap Templates](tasks/updating-machine-templates.md)
    - [Workload bootstrap using GitOps](tasks/workload-bootstrap-gitops.md)
    - [Automated Machine management](./tasks/automated-machine-management/index.md)
      - [Scaling](./tasks/automated-machine-management/scaling.md)
      - [Autoscaling](./tasks/automated-machine-management/autoscaling.md)
      - [Healthchecking](./tasks/automated-machine-management/healthchecking.md)
    - [Experimental Features](./tasks/experimental-features/experimental-features.md)
        - [MachinePools](./tasks/experimental-features/machine-pools.md)
        - [MachineSetPreflightChecks](./tasks/experimental-features/machineset-preflight-checks.md)
        - [ClusterResourceSet](./tasks/experimental-features/cluster-resource-set.md)
        - [ClusterClass](./tasks/experimental-features/cluster-class/index.md)
            - [Writing a ClusterClass](./tasks/experimental-features/cluster-class/write-clusterclass.md)
            - [Changing a ClusterClass](./tasks/experimental-features/cluster-class/change-clusterclass.md)
            - [Operating a managed Cluster](./tasks/experimental-features/cluster-class/operate-cluster.md)
        - [Runtime SDK](tasks/experimental-features/runtime-sdk/index.md)
            - [Implementing Runtime Extensions](./tasks/experimental-features/runtime-sdk/implement-extensions.md)
            - [Implementing Lifecycle Hook Extensions](./tasks/experimental-features/runtime-sdk/implement-lifecycle-hooks.md)
            - [Implementing Topology Mutation Hook Extensions](./tasks/experimental-features/runtime-sdk/implement-topology-mutation-hook.md)
            - [Deploying Runtime Extensions](./tasks/experimental-features/runtime-sdk/deploy-runtime-extension.md)
        - [Ignition Bootstrap configuration](./tasks/experimental-features/ignition.md)
    - [Running multiple providers](./tasks/multiple-providers.md)
    - [Verification of Container Images](./tasks/verify-container-images.md)
    - [Diagnostics](./tasks/diagnostics.md)
- [Security Guidelines](./security/index.md)
    - [Pod Security Standards](./security/pod-security-standards.md)
- [clusterctl CLI](./clusterctl/overview.md)
    - [clusterctl Commands](clusterctl/commands/commands.md)
        - [init](clusterctl/commands/init.md)
        - [generate cluster](clusterctl/commands/generate-cluster.md)
        - [generate provider](clusterctl/commands/generate-provider.md)
        - [generate yaml](clusterctl/commands/generate-yaml.md)
        - [get kubeconfig](clusterctl/commands/get-kubeconfig.md)
        - [describe cluster](clusterctl/commands/describe-cluster.md)
        - [move](./clusterctl/commands/move.md)
        - [upgrade](clusterctl/commands/upgrade.md)
        - [delete](clusterctl/commands/delete.md)
        - [completion](clusterctl/commands/completion.md)
        - [alpha rollout](clusterctl/commands/alpha-rollout.md)
        - [alpha topology plan](clusterctl/commands/alpha-topology-plan.md)
        - [additional commands](clusterctl/commands/additional-commands.md)
    - [clusterctl Configuration](clusterctl/configuration.md)
    - [clusterctl Provider Contract](clusterctl/provider-contract.md)
    - [clusterctl for Developers](clusterctl/developers.md)
    - [clusterctl Extensions with Plugins](clusterctl/plugins.md)
- [Developer Guide](./developer/guide.md)
    - [Repository Layout](./developer/repository-layout.md)
    - [Rapid iterative development with Tilt](./developer/tilt.md)
    - [Logging](./developer/logging.md)
    - [Testing](./developer/testing.md)
    - [Developing E2E tests](./developer/e2e.md)
    - [Controllers](./developer/architecture/controllers.md)
        - [Bootstrap](./developer/architecture/controllers/bootstrap.md)
        - [Cluster](./developer/architecture/controllers/cluster.md)
        - [Machine](./developer/architecture/controllers/machine.md)
        - [MachineSet](./developer/architecture/controllers/machine-set.md)
        - [MachineDeployment](./developer/architecture/controllers/machine-deployment.md)
        - [MachineHealthCheck](./developer/architecture/controllers/machine-health-check.md)
        - [Control Plane](./developer/architecture/controllers/control-plane.md)
        - [MachinePool](./developer/architecture/controllers/machine-pool.md)
        - [ClusterTopology](./developer/architecture/controllers/cluster-topology.md)
        - [ClusterResourceSet](./developer/architecture/controllers/cluster-resource-set.md)
        - [Metadata propagation](./developer/architecture/controllers/metadata-propagation.md)
        - [Multi-tenancy](./developer/architecture/controllers/multi-tenancy.md)
        - [Support multiple instances](./developer/architecture/controllers/support-multiple-instances.md)
        - [Tuning controllers](./developer/architecture/controllers/tuning.md)
    - [Provider Implementers](./developer/providers/implementers.md)
        - [Version migration](./developer/providers/version-migration.md)
          - [v0.3 to v0.4](./developer/providers/migrations/v0.3-to-v0.4.md)
          - [v0.4 to v1.0](./developer/providers/migrations/v0.4-to-v1.0.md)
          - [v1.0 to v1.1](./developer/providers/migrations/v1.0-to-v1.1.md)
          - [v1.1 to v1.2](./developer/providers/migrations/v1.1-to-v1.2.md)
          - [v1.2 to v1.3](./developer/providers/migrations/v1.2-to-v1.3.md)
          - [v1.3 to v1.4](./developer/providers/migrations/v1.3-to-v1.4.md)
          - [v1.4 to v1.5](./developer/providers/migrations/v1.4-to-v1.5.md)
          - [v1.5 to v1.6](./developer/providers/migrations/v1.5-to-v1.6.md)
          - [v1.6 to v1.7](./developer/providers/migrations/v1.6-to-v1.7.md)
        - [Provider contracts](./developer/providers/contracts.md)
          - [Cluster Infrastructure](./developer/providers/cluster-infrastructure.md)
          - [Machine Infrastructure](./developer/providers/machine-infrastructure.md)
          - [Bootstrap](./developer/providers/bootstrap.md)
        - [Implementer's Guide](./developer/providers/implementers-guide/overview.md)
          - [Naming](./developer/providers/implementers-guide/naming.md)
          - [Configure](./developer/providers/implementers-guide/configure.md)
          - [Create Repo and Generate CRDs](./developer/providers/implementers-guide/generate_crds.md)
          - [Create API](./developer/providers/implementers-guide/create_api.md)
          - [Webhooks](./developer/providers/webhooks.md)
          - [Controllers and Reconciliation](./developer/providers/implementers-guide/controllers_and_reconciliation.md)
          - [Building, Running, Testing](./developer/providers/implementers-guide/building_running_and_testing.md)
    - [CustomResourceDefinitions relationships](./developer/crd-relationships.md)
- [Troubleshooting](./user/troubleshooting.md)
- [Reference](./reference/reference.md)
    - [API Reference](./reference/api_reference.md)
    - [Glossary](./reference/glossary.md)
    - [Provider List](./reference/providers.md)
    - [Ports](./reference/ports.md)
    - [Code of Conduct](./code-of-conduct.md)
    - [Contributing](./CONTRIBUTING.md)
    - [Jobs](./reference/jobs.md)
    - [Code Review in Cluster API](./REVIEWING.md)
    - [Version Support](./reference/versions.md)
    - [Supported Labels and Annotations](./reference/labels_and_annotations.md)
    - [Owner References](./reference/owner_references.md)
