# Cluster API v1.0 compared to v1.1

This document provides an overview over relevant changes between ClusterAPI v1.0 and v1.1 for
maintainer of other providers and consumers of our Go API.

## Minimum Go version

* The Go version used by Cluster API is now Go 1.17+

## Dependencies

**Note**: Only the most relevant dependencies are listed, `k8s.io/` and `ginkgo`/`gomega` dependencies
are kept in sync with the versions used by `sigs.k8s.io/controller-runtime`. 

- sigs.k8s.io/controller-runtime: v0.10.x => v0.11.x
- k8s.io/*: v0.22.x => v0.23.x (derived from controller-runtime)
- github.com/go-logr/logr: v0.4.0 => v1.2.0 (derived from controller-runtime)
- k8s.io/klog/v2: v2.9.0 => v2.30.0 (derived from controller-runtime)
- sigs.k8s.io/controller-tools: v0.7.x => v0.8.x
- sigs.k8s.io/kind: v0.11.x => v0.11.x

## Changes by Kind

### Deprecation

* The Cluster and ClusterClass webhooks have been moved to the `webhooks` package. Thus, the following methods on `Cluster` and `ClusterClass` 
  in `api/v1beta1` are deprecated: `SetupWebhookWithManager`, `Default`, `ValidateCreate`, `ValidateUpdate` and `ValidateDelete`.
* The `third_party/kubernetes-drain` package is deprecated, as we're now using `k8s.io/kubectl/pkg/drain` instead ([PR](https://github.com/kubernetes-sigs/cluster-api/pull/5440)). 
* `util/version.CompareWithBuildIdentifiers` has been deprecated, please use `util/version.Compare(a, b, WithBuildTags())` instead.
* The functions `annotations.HasPausedAnnotation` and `annotations.HasSkipRemediationAnnotation` have been deprecated, please use
  `annotations.HasPaused` and `annotations.HasSkipRemediation` respectively instead.

### Removals

* `KCPUpgradeSpec` has been removed. Please use `ClusterUpgradeConformanceSpec` instead.

### API Change

* Some controllers have been moved to internal to reduce their API surface. We now only
  surface what is necessary, e.g. the reconciler and the `SetupWithManager` func:
    * [bootstrap/kubeadm/controllers](https://github.com/kubernetes-sigs/cluster-api/pull/5493)
    * controllers [part1](https://github.com/kubernetes-sigs/cluster-api/pull/5899) [part2](https://github.com/kubernetes-sigs/cluster-api/pull/5913) [part3](https://github.com/kubernetes-sigs/cluster-api/pull/5917)
    * [controlplane/kubeadm/controllers](https://github.com/kubernetes-sigs/cluster-api/pull/5929)
    * [exp/addons/controllers](https://github.com/kubernetes-sigs/cluster-api/pull/5639) 
    * [test/infrastructure/docker/controllers](https://github.com/kubernetes-sigs/cluster-api/pull/5595) 
    * [test/infrastructure/docker/exp/controllers](https://github.com/kubernetes-sigs/cluster-api/pull/5690) 
    * [exp/controllers](https://github.com/kubernetes-sigs/cluster-api/pull/5651)

* Following packages have been moved to internal
    * [/test/infrastructure/docker/cloudinit](https://github.com/kubernetes-sigs/cluster-api/pull/5795)
    * [/test/infrastructure/docker/docker](https://github.com/kubernetes-sigs/cluster-api/pull/5795)
    * [/test/infrastructure/docker/third_party](https://github.com/kubernetes-sigs/cluster-api/pull/5795)
    * [/test/infrastructure/docker/exp/docker](https://github.com/kubernetes-sigs/cluster-api/pull/5795)

### Other

* ClusterClass:
    * `clusterctl` is now able to handle cluster templates with ClusterClasses ([PR](https://github.com/kubernetes-sigs/cluster-api/pull/5351)).
      Please check out the corresponding documentation in [clusterctl provider contract](../../../clusterctl/provider-contract.md#clusterclass-definitions)
      If you have any further questions about writing ClusterClasses, please let us know.
    * e2e tests:
        * `QuickStartSpec` is now able to test clusters using ClusterClass. Please see this [PR](https://github.com/kubernetes-sigs/cluster-api/pull/5423) for an example on how to use it.
        * `SelfHostedSpec` is now able to test clusters using ClusterClass. Please see this [PR](https://github.com/kubernetes-sigs/cluster-api/pull/5600) for an example on how to use it.
* Test framework provides better logging in case of failures when creating the bootstrap kind cluster; in order to
  fully exploit this feature, it is required to pass the `LogFolder` parameter when calling `CreateKindBootstrapClusterAndLoadImages`. Please see this [PR](https://github.com/kubernetes-sigs/cluster-api/pull/5910) for an example on how to use it. 
* The `gci` linter has been enabled to enforce consistent imports. As usual, feel free to take a look at our linter config, but of course it's not mandatory to adopt it.
* The Tilt dev setup has been extended with:
    * [an option to deploy Grafana, Loki and Promtail](https://github.com/kubernetes-sigs/cluster-api/pull/5336) 
    * [options to ease debugging of controllers (delve, profiler, metrics)](https://github.com/kubernetes-sigs/cluster-api/pull/5485) 
